# -*- coding: utf-8 -*-
"""
Created on Fri Mar  8 14:33:26 2024

@author: jschi
"""

from datetime import datetime


def is_iso8601(string):
    iso_format = "%Y-%m-%dT%H:%M:%S"
    try:
        datetime.strptime(string, iso_format)
        return True
    except ValueError:
        return False


def parse_args_to_dict(args):
    args_dict = {}
    for arg in args:
        # Split the argument on '=' which separates the parameter from its value
        if '=' in arg:
            key, val = arg.split('=', 1)
            # Optionally remove leading dashes from parameters
            key = key.lstrip('-')
            args_dict[key] = val
    return args_dict



# -*- coding: utf-8 -*-

class Person:
    """This class represents a German parliamentarian"""

    def __init__(self, dictionary):
        self.btid = dictionary["id"]
        if "nachname" in dictionary:
            self.lastname = dictionary["nachname"]
        else:
            self.lastname = None
        if "vorname" in dictionary:
            self.firstname = dictionary["vorname"]
        else:
            self.firstname = None
        mdbrole = False
        if "basisdatum" in dictionary:
            self.basedate = dictionary["basisdatum"]
        else:
            self.basedate = None
        if "datum" in dictionary:
            self.date = dictionary["datum"]
        else:
            self.date = None
        if "namenszusatz" in dictionary:
            self.nameaddendum = dictionary["namenszusatz"]
        ttl = dictionary["titel"].split(",")
        if ttl[1] == " MdB":
            self.faction = ttl[2].strip()
            mdbrole = True
        elif "person_roles" in dictionary:
            if "fraktion" in dictionary["person_roles"][0]:
                self.faction = dictionary["person_roles"][0]["fraktion"]
        else:
            self.faction = None
        if ttl[0].split(dictionary["vorname"])[0] != "":
            self.titel = ttl[0].split(dictionary["vorname"])[0].strip()
        else:
            self.titel = None
        if "wahlperiode" in dictionary:
            self.legislativeperiod = dictionary["wahlperiode"]
        else:
            self.legislativeperiod = None
        if "person_roles" in dictionary:
            liro = []
            for r in dictionary["person_roles"]:
                liro.append(Role(r))
            self.roles = liro
        elif "person_roles" not in dictionary and mdbrole is True and "wahlperiode" in dictionary:
            self.roles = [Role({"funktion": "MdB",
                               "fraktion": self.faction,
                                "nachname": self.lastname,
                                "vorname": self.firstname,
                                "wahlperiode_nummer": self.legislativeperiod
                                })]
        else:
            self.roles = None
        if "wahlperiode" in dictionary:
            self.legislativeperiod = dictionary["wahlperiode"]
        else:
            self.legislativeperiod = None

    def returnroles(self):
        for r in self.roles:
            print(r.returnrole())

    def update_by_id(self, apikey=None):
        if apikey is None:
            raise ValueError("Function needs an API key.")
        dat1 = self.query(apikey, resource="person", fid=self.btid)
        dictionary = dat1["documents"][0]
        mdbrole = False
        if "basisdatum" in dictionary:
            self.basedate = dictionary["basisdatum"]
        else:
            self.basedate = None
        if "datum" in dictionary:
            self.date = dictionary["datum"]
        else:
            self.date = None
        if "namenszusatz" in dictionary:
            self.nameaddendum = dictionary["namenszusatz"]
        ttl = dictionary["titel"].split(",")
        if ttl[1] == " MdB":
            self.faction = ttl[2].strip()
            mdbrole = True
        elif "person_roles" in dictionary:
            if "fraktion" in dictionary["person_roles"][0]:
                self.faction = dictionary["person_roles"][0]["fraktion"]
        else:
            self.faction = None
        if ttl[0].split(dictionary["vorname"])[0] != "":
            self.titel = ttl[0].split(dictionary["vorname"])[0].strip()
        else:
            self.titel = None
        if "wahlperiode" in dictionary:
            self.legislativeperiod = dictionary["wahlperiode"]
        else:
            self.legislativeperiod = None
        if "person_roles" in dictionary:
            liro = []
            for r in dictionary["person_roles"]:
                liro.append(Role(r))
            self.roles = liro
        elif "person_roles" not in dictionary and mdbrole is True and "wahlperiode" in dictionary:
            self.roles = [Role({"funktion": "MdB",
                               "fraktion": self.faction,
                                "nachname": self.lastname,
                                "vorname": self.firstname,
                                "wahlperiode_nummer": self.legislativeperiod
                                })]
        else:
            self.roles = None
        if "wahlperiode" in dictionary:
            self.legislativeperiod = dictionary["wahlperiode"]
        else:
            self.legislativeperiod = None


class Role:
    """This class presents a role in the German parliamentary system."""

    def __init__(self, dictionary):
        self.function = dictionary["funktion"]
        if "wahlperiode_nummer" in dictionary:
            self.legislativeperiod = dictionary["wahlperiode_nummer"]
        else:
            self.legislativeperiod = None
        if "namenszusatz" in dictionary:
            self.nameaddendum = dictionary["namenszusatz"]
        else:
            self.nameaddendum = None
        if "funktionszusatz" in dictionary:
            self.functionaddendum = dictionary["funktionszusatz"]
        else:
            self.functionaddendum = None
        if "fraktion" in dictionary:
            self.faction = dictionary["fraktion"]
        else:
            self.faction = None
        if "bundesland" in dictionary:
            self.federalstate = dictionary["bundesland"]
        else:
            self.federalstate = None
        if "nachname" in dictionary:
            self.lastname = dictionary["nachname"]
        else:
            self.lastname = None
        if "vorname" in dictionary:
            self.firstname = dictionary["vorname"]
        else:
            self.firstname = None
        if "wahlkreiszusatz" in dictionary:
            self.districtaddendum = dictionary["wahlkreiszusatz"]
        else:
            self.districtaddendum = None
        if "ressort_titel" in dictionary:
            self.department = dictionary["ressort_titel"]
        else:
            self.department = None

    def __str__(self):
        return f'Person: {self.firstname}{" " if self.nameaddendum!=None else ""}{self.nameaddendum if self.nameaddendum!=None else ""} {self.lastname} {"(" if self.faction!= None else ""}{self.faction if self.faction!= None else ""}{")" if self.faction!= None else ""} - {self.function}'

    def __repr__(self):
        return f'Person: {self.firstname}{" " if self.nameaddendum!=None else ""}{self.nameaddendum if self.nameaddendum!=None else ""} {self.lastname} {"(" if self.faction!= None else ""}{self.faction if self.faction!= None else ""}{")" if self.faction!= None else ""} - {self.function}'

    def returnrole(self):
        return (
            f'{self.firstname}{" " if self.nameaddendum!=None else ""}{self.nameaddendum if self.nameaddendum!=None else ""} {self.lastname} {"(" if self.faction!= None else ""}{self.faction if self.faction!= None else ""}{")" if self.faction!= None else ""},'
            f'{self.function}{" - " if self.functionaddendum!= None else ""}{self.functionaddendum if self.functionaddendum!= None else ""}{", " if self.department!= None else ""}{self.department if self.department!= None else ""}{" (" if self.federalstate!=None else ""}{self.federalstate if self.federalstate!=None else ""}{")" if self.federalstate!=None else ""}')


class Drucksache:
    """This class represents a document of the German federal parliaments"""

    def __init__(self, dictionary):
        self.btid = int(dictionary["id"])
        if "herausgeber" in dictionary:
            if dictionary["herausgeber"] == "BT":
                self.publisher = "Bundestag"
            elif dictionary["herausgeber"] == "BR":
                self.publisher = "Bundesrat"
            elif dictionary["herausgeber"] is not None and dictionary["herausgeber"] != "BR" and dictionary["herausgeber"] != "BT":
                self.publisher = dictionary["herausgeber"]
        else:
            self.publisher = None
        if "urheber" in dictionary:
            self.originator = dictionary["urheber"]
        else:
            self.originator = None
        if "autoren_anzahl" in dictionary:
            self.author_nr = dictionary["autoren_anzahl"]
        else:
            self.author_nr = None
        if "ressort" in dictionary:
            self.author_nr = dictionary["ressort"]
        else:
            self.author_nr = None
        if "datum" in dictionary:
            self.date = dictionary["datum"]
        else:
            self.date = None
        if "wahlperiode" in dictionary:
            self.legislativeperiod = dictionary["wahlperiode"]
        else:
            self.legislativeperiod = None
        if "titel" in dictionary:
            self.title = dictionary["titel"]
        else:
            self.title = None
        if "drucksachetyp" in dictionary:
            self.doctype = dictionary["drucksachetyp"]
        else:
            self.doctype = None
        if "fundstelle" in dictionary:
            if "pdf_url" in dictionary["fundstelle"]:
                self.pdf_url = dictionary["fundstelle"]["pdf_url"]
            self.reference = dictionary["fundstelle"]
        else:
            self.reference = None
        if "dokumentart" in dictionary:
            self.docname = dictionary["dokumentart"]
        else:
            self.docname = None
        if "typ" in dictionary:
            self.instance = dictionary["typ"]
        else:
            self.instance = None
        if "dokumentnummer" in dictionary:
            self.docnumber = dictionary["dokumentnummer"]
        else:
            self.docnumber = None
        if "autoren_anzeige" in dictionary:
            auan = []
            auanid = []
            for a in dictionary["autoren_anzeige"]:
                auan.append(a["titel"])
                auanid.append(a["id"])
            self.author = auan
            self.authorid = auanid
            self.authordisplay = dictionary["autoren_anzeige"]
        else:
            self.author = None
            self.authordisplay = None
        if "text" in dictionary:
            self.text = dictionary["text"]
        else:
            self.text = None

    def __str__(self):
        return f'{self.instance}: ({self.btid}) {self.doctype} - {self.title} - {self.date}'

    def __repr__(self):
        return f'{self.instance}: ({self.btid}) {self.doctype} - {self.title} - {self.date}'

    def get_authors(self, apikey):
        pass


class Aktivitaet:
    """This class represents an activity in the German federal parliaments"""

    def __init__(self, dictionary):
        self.btid = dictionary["id"]
        self.activitytype = dictionary["aktivitaetsart"]
        self.date = dictionary["datum"]
        self.title = dictionary["titel"]
        self.type = dictionary["typ"]
        self.doctype = dictionary["dokumentart"]
        self.parlsession = dictionary["wahlperiode"]
        self.numprocedure = dictionary["vorgangsbezug_anzahl"]
        self.procedure_reference = dictionary["vorgangsbezug"][0]["id"]
        self.document_reference = dictionary["fundstelle"]["id"]

    def __str__(self):
        return f'{self.instance}: ({self.btid}) {self.activitytype} - {self.title} - {self.date}'

    def get_procedure(self, apikey):
        data = self.query(apikey=apikey, resource="vorgang",
                          fid=self.procedure_reference)
        data = Vorgang(data[0])
        return data

    def get_document(self, apikey):
        data = self.query(
            apikey=apikey, resource="drucksache-text", fid=self.document_reference)
        data = Drucksache(data[0])
        return data


class Vorgang:
    """This class represents a legislative process in of the German federal parliaments"""

    def __init__(self, dictionary):
        self.btid = dictionary["id"]
        self.process_positions = []
        if "datum" in dictionary:
            self.date = dictionary["datum"]
        else:
            self.date = None
        if "titel" in dictionary:
            self.title = dictionary["titel"]
        else:
            self.title = None
        if "typ" in dictionary:
            self.instance = dictionary["typ"]
        else:
            self.instance = None
        if "vorgangstyp" in dictionary:
            self.processtype = dictionary["vorgangstyp"]
        else:
            self.processtype = None
        if "initiative" in dictionary:
            self.initiativ = dictionary["initiative"]
        else:
            self.initiativ = None
        if "abstract" in dictionary:
            self.abstract = dictionary["abstract"]
        else:
            self.abstract = None
        if "archiv" in dictionary:
            self.archive = dictionary["archiv"]
        else:
            self.archive = None
        if "beratungsstand" in dictionary:
            self.status = dictionary["beratungsstand"]
        else:
            self.status = None
        if "deskriptor" in dictionary:
            self.descriptor = dictionary["deskriptor"]
        else:
            self.descriptor = None
        if "gesta" in dictionary:
            self.gesta = dictionary["gesta"]
        else:
            self.gesta = None
        if "inkrafttreten" in dictionary:
            self.effectivedate = dictionary["inkrafttreten"][0]["datum"]
        else:
            self.effectivedate = None
        if "kom" in dictionary:
            self.kom = dictionary["kom"]
        else:
            self.kom = None
        if "mitteilung" in dictionary:
            self.notification = dictionary["mitteilung"]
        else:
            self.notification = None
        if "ratsdok" in dictionary:
            self.eucouncilnr = dictionary["ratsdok"]
        else:
            self.eucouncilnr = None
        if "sachgebiet" in dictionary:
            self.subject = dictionary["sachgebiet"]
        else:
            self.subject = None
        if "verkuendung" in dictionary:
            self.announcement = dictionary["verkuendung"]
        else:
            self.announcement = None
        if "wahlperiode" in dictionary:
            self.legislativeperiod = dictionary["wahlperiode"]
        else:
            self.legislativeperiod = None
        if "zustimmungsbeduerftigkeit" in dictionary:
            self.approvalnecessary = dictionary["zustimmungsbeduerftigkeit"]
            self.approvalnecessaryBool = dictionary["zustimmungsbeduerftigkeit"][len(
                dictionary["zustimmungsbeduerftigkeit"])-1].split(",")[0]
            if any("bes.eilbed." in s for s in dictionary["zustimmungsbeduerftigkeit"]):
                self.urgency = True
        else:
            self.approvalnecessary = None
            self.approvalnecessaryBool = None
            self.urgency = None

    def get_positions(self):
        data = self.query("vorgangsposition", procedureID=self.btid)
        for d in data:
            self.process_positions.append(Vorgangsposition(d))

    def show_positions(self):
        for pp in self.process_positions:
            print(pp)

    def __str__(self):
        return f'{self.instance}: ({self.btid}) {self.processtype} - {self.title} - {self.date}'

    def __repr__(self):
        return f'{self.instance}: ({self.btid}) {self.processtype} - {self.title} - {self.date}'


class Vorgangsposition:
    """This class represents a step in a legislative process in the German federal parliaments"""

    def __init__(self, dictionary):
        self.btid = dictionary["id"]
        if "aktivitaet_anzeige" in dictionary:
            pass
        if "datum" in dictionary:
            self.date = dictionary["datum"]
        else:
            self.date = None
        if "dokumentart" in dictionary:
            self.docname = dictionary["dokumentart"]
        else:
            self.docname = None
        if "fortsetzung" in dictionary:
            self.continuation = dictionary["fortsetzung"]
        else:
            self.continuation = None
        if "gang" in dictionary:
            self.course = dictionary["gang"]
        else:
            self.course = None
        if "gang" in dictionary:
            self.course = dictionary["gang"]
        else:
            self.course = None
        if "nachtrag" in dictionary:
            self.Supplement = dictionary["nachtrag"]
        else:
            self.Supplement = None
        if "titel" in dictionary:
            self.title = dictionary["titel"]
        else:
            self.title = None
        if "typ" in dictionary:
            self.instance = dictionary["typ"]
        else:
            self.instance = None
        if "urheber" in dictionary:
            self.originator = dictionary["urheber"]
        else:
            self.originator = None
        if "vorgang_id" in dictionary:
            self.procedureID = dictionary["vorgang_id"]
        else:
            self.procedureID = None
        if "vorgangsposition" in dictionary:
            self.processposition = dictionary["vorgangsposition"]
        else:
            self.processposition = None
        if "vorgangstyp" in dictionary:
            self.processtype = dictionary["vorgangstyp"]
        else:
            self.processtype = None
        if "zuordnung" in dictionary:
            if dictionary["zuordnung"] == "BT":
                self.institution = "Bundestag"
            elif dictionary["zuordnung"] == "BR":
                self.institution = "Bundesrat"
            elif dictionary["zuordnung"] is not None and dictionary["zuordnung"] != "BR" and dictionary["zuordnung"] != "BT":
                self.institution = dictionary["zuordnung"]
        else:
            self.institution = None

    def __str__(self):
        return f'{self.instance}: ({self.procedureID}) {self.processtype} - {self.title} - {self.date}'

    def __repr__(self):
        pass


class Plenarprotokoll:
    """This class represents a plenary protocol of the German federal parliaments"""

    def __init__(self, dictionary):
        self.btid = dictionary["id"]
        if "datum" in dictionary:
            self.date = dictionary["datum"]
        else:
            self.date = None
        if "dokumentart" in dictionary:
            self.docname = dictionary["dokumentart"]
        else:
            self.docname = None
        if "titel" in dictionary:
            self.title = dictionary["titel"]
        else:
            self.title = None
        if "typ" in dictionary:
            self.instance = dictionary["typ"]
        else:
            self.instance = None
        if "herausgeber" in dictionary:
            if dictionary["herausgeber"] == "BT":
                self.publisher = "Bundestag"
            elif dictionary["herausgeber"] == "BR":
                self.publisher = "Bundesrat"
            elif dictionary["herausgeber"] is not None and dictionary["herausgeber"] != "BR" and dictionary["herausgeber"] != "BT":
                self.publisher = dictionary["herausgeber"]
        else:
            self.publisher = None
        if "wahlperiode" in dictionary:
            self.legislativeperiod = dictionary["wahlperiode"]
        else:
            self.legislativeperiod = None
        if "text" in dictionary:
            self.text = dictionary["text"]
        else:
            self.text = None
        if "fundstelle" in dictionary:
            if "pdf_url" in dictionary["fundstelle"]:
                self.pdf_url = dictionary["fundstelle"]["pdf_url"]
            self.reference = dictionary["fundstelle"]
        else:
            self.reference = None
        if "sitzungsbemerkung" in dictionary:
            self.sessioncomment = dictionary["sitzungsbemerkung"]
        else:
            self.sessioncomment = None
        if "dokumentnummer" in dictionary:
            self.docnumber = dictionary["dokumentnummer"]
        else:
            self.docnumber = None

    def __str__(self):
        return f'{self.docname}: {self.docnumber} - {self.title} - {self.date}'

    def __repr__(self):
        pass



# -*- coding: utf-8 -*-

import pytest
from datetime import datetime
import bundestag_api
from random import randrange


def checkdates(date, dateL, dateE=None):
    date = datetime.strptime(date, "%Y-%m-%d")
    dateL = datetime.strptime(dateL, "%Y-%m-%d")
    if dateE is not None:
        dateE = datetime.strptime(dateE, "%Y-%m-%d")
        if date <= dateL and date >= dateE:
            return True
        else:
            return False
    if date <= dateL:
        return True
    else:
        return False


def test_checkdates():
    date = "2022-05-11"
    dateL = "2022-05-15"
    dateE = "2022-05-01"
    erl = []
    if checkdates(date, dateL) == True:
        erl.append(False)
    else:
        erl.append(True)
    if checkdates(date, dateL, dateE=dateE) == True:
        erl.append(False)
    else:
        erl.append(True)
    assert len(erl) == 2 and not any(erl)


def test_genapikey():
    bta = bundestag_api.btaConnection()
    assert bta.apikey == "GmEPb1B.bfqJLIhcGAsH9fTJevTglhFpCoZyAAAdhp"


def test_wrongapikey():
    bta2 = bundestag_api.btaConnection("GmEPb1B.bfqJLIhcGAsH9fTJevTglhFpCoZyAArrhp")
    test = bta2.search_document()
    assert test == 'An authorization error occured. Likely an error with you API key. Code 401: Unauthorized'


@pytest.fixture
def bta_object():
    bta = bundestag_api.btaConnection()
    return bta


@pytest.mark.parametrize("resourcetypes", ["aktivitaet", "drucksache", "drucksache-text", "person",
                                           "plenarprotokoll", "plenarprotokoll-text", "vorgang",
                                           "vorgangsposition"])
def test_query_resources(resourcetypes):
    bta = bundestag_api.btaConnection()
    data = bta.query(resource=resourcetypes)
    assert isinstance(data, list)


def test_query_resources_num(resourcetypes):
    bta = bundestag_api.btaConnection()
    data = bta.query(resource=resourcetypes, num=150)
    assert len(data) == 150


def test_query_docID():
    bta = bundestag_api.btaConnection()
    data = bta.query(resource="vorgang", documentID=259520)
    assert data["id"] == "284323"


def test_query_docID_fail():
    with pytest.raises(ValueError):
        bta = bundestag_api.btaConnection()
        data = bta.query(resource="drucksache", documentID=259520)


def test_query_resources_noreturn(bta_object):
    bta = bta_object
    data = bta.query(resource="vorgang", dateend="2050-01-01", datestart="2050-01-01")
    assert data == "No data was returned."


def test_getdocument(bta_object):
    bta = bta_object
    data = bta.get_document(btid=264026)
    assert data["titel"] == "Rettungsschirm für Spitzenforschungseinrichtungen"


def test_getdocument_object(bta_object):
    bta = bta_object
    data = bta.get_document(btid=264026, rformat="object")
    assert data.doctype == "Kleine Anfrage"


with pytest.raises(TypeError):
    bta = bundestag_api.btaConnection()
    data = bta.get_document(btid="264026")


def test_searchplenary(bta_object):
    bta = bta_object
    erl = []
    data = bta.search_plenaryprotocol(num=50)
    if len(data) == 50:
        erl.append("False")
    else:
        erl.append("True")
    data = bta.search_plenaryprotocol(num=10, datestart="2022-05-01", dateend="2022-05-11")
    if len(data) == 10:
        erl.append("False")
    else:
        erl.append("True")
    erl.append(checkdates(data[randrange(10)]["datum"], "2022-05-11", dateE="2022-05-01"))
    data = bta.search_plenaryprotocol(num=10, rformat="object")
    tl = list(data.keys())
    if type(data[tl[randrange(9)]]) == bundestag_api.bta_wrapper.Plenarprotokoll:
        erl.append("False")
    else:
        erl.append("True")
    data = bta.search_plenaryprotocol(num=10, rformat="json", institution="BR")
    if data[randrange(10)]["herausgeber"] == "BR":
        erl.append("False")
    else:
        erl.append("True")
    assert len(erl) == 5 and not any(erl)


def test_getplenary(bta_object):
    bta = bta_object
    erl = []
    data = bta.get_plenaryprotocol(btid=5511)
    if data["dokumentnummer"] == "1024":
        erl.append("False")
    else:
        erl.append("True")
    data = bta.get_plenaryprotocol(btid=5511, rformat="object")
    tl = list(data.keys())
    if type(data[tl[randrange(9)]]) == bundestag_api.bta_wrapper.Plenarprotokoll:
        erl.append("False")
    else:
        erl.append("True")
    assert len(erl) == 2 and not any(erl)


def test_searchactivity(bta_object):
    bta = bta_object
    erl = []
    data = bta.search_activity(num=50)
    if len(data) == 50 and data[0]["typ"] == "Aktivität":
        erl.append("False")
    else:
        erl.append("True")
    data = bta.search_activity(num=10, datestart="2022-05-01", dateend="2022-05-11")
    if len(data) == 10:
        erl.append("False")
    else:
        erl.append("True")
    erl.append(checkdates(data[randrange(10)]["datum"], "2022-05-11", dateE="2022-05-01"))
    data = bta.search_activity(num=10, rformat="object")
    tl = list(data.keys())
    if type(data[tl[randrange(9)]]) == bundestag_api.bta_wrapper.Aktivitaet:
        erl.append("False")
    else:
        erl.append("True")
    data = bta.search_activity(num=10, rformat="json", institution="BT")
    if data[randrange(10)]["herausgeber"] == "BT":
        erl.append("False")
    else:
        erl.append("True")
    assert len(erl) == 5 and not any(erl)


def test_getactivity(bta_object):
    bta = bta_object
    erl = []
    data = bta.get_activity(btid=1618465)
    if data["aktivitaetsart"] == "Kleine Anfrage":
        erl.append("False")
    else:
        erl.append("True")
    data = bta.get_activity(btid=5511, rformat="object")
    tl = list(data.keys())
    if type(data[tl[0]]) == bundestag_api.bta_wrapper.Aktivitaet:
        erl.append("False")
    else:
        erl.append("True")
    assert len(erl) == 2 and not any(erl)


def test_searchperson(bta_object):
    bta = bta_object
    erl = []
    data = bta.search_person(num=50)
    if len(data) == 50 and data[0]["typ"] == "Person":
        erl.append("False")
    else:
        erl.append("True")
    data = bta.search_person(rformat="object")
    tl = list(data.keys())
    if type(data[tl[0]]) == bundestag_api.bta_wrapper.Person:
        erl.append("False")
    else:
        erl.append("True")
    assert len(erl) == 2 and not any(erl)


def test_getperson(bta_object):
    bta = bta_object
    erl = []
    data = bta.get_person(btid=7302)
    if data["nachname"] == "Hennig-Wellsow":
        erl.append("False")
    else:
        erl.append("True")
    data = bta.get_person(btid=7302, rformat="object")
    if type(data) == bundestag_api.bta_wrapper.Person:
        erl.append("False")
    else:
        erl.append("True")
    assert len(erl) == 2 and not any(erl)



# -*- coding: utf-8 -*-

import bundestag_api

bta = bundestag_api.btaConnection()

test = bta.btapi_query(resource="drucksache")


data = bta.get_document(btid=264035)

resourcetypes = ["aktivitaet", "drucksache", "drucksache-text", "person",
                 "plenarprotokoll", "plenarprotokoll-text", "vorgang",
                 "vorgangsposition"]
bta = bundestag_api.BTA_Connection("GmEPb1B.bfqJLIhcGAsH9fTJevTglhFpCoZyAArrhp")
data = bta.query(resource="vorgang")
data2 = bta.query(resource="vorgang", rformat="object")
dat2 = bta.query(resource="person", rformat="object")

for r in resourcetypes:
    dat2 = bta.query(resource=r, rformat="object")



# -*- coding: utf-8 -*-
"""
Python wrapper for the official Bundestag API
"""

from .bta_wrapper import btaConnection
from .models import Person, Role, Drucksache, Aktivitaet, Vorgang, Vorgangsposition, Plenarprotokoll



# -*- coding: utf-8 -*-
from datetime import datetime
import requests
import sys
import logging
from models import Person, Aktivitaet, Vorgang, Vorgangsposition, Drucksache, Plenarprotokoll
from utils import is_iso8601, parse_args_to_dict

logger = logging.getLogger("bundestag_api")
logger.addHandler(logging.NullHandler())


class btaConnection:
    """This class handles the API authentication and provides search functionality

    Methods
    -------
    query(resource, return_format="json", num=100, fid=None, date_start=None, date_end=None,
          institution=None, documentID=None, plenaryprotocolID=None, procedureID=None)
        A general search function for the official Bundestag API
    search_procedure(return_format="json",num=100,fid=None,date_start=None,date_end=None):
        Searches procedures specified by the parameters
    search_procedureposition(return_format="json", num=100, fid=None, date_start=None, date_end=None, procedureID=None):
        Searches procedure positions specified by the parameters
    search_document(return_format="json", num=100, fid=None, date_start=None, date_end=None,institution=None):
        Searches documents specified by the parameters
    search_person(return_format="json", num=100, fid=None):
        Searches persons specified by the parameters
    search_plenaryprotocol(return_format="json", num=100, fid=None, date_start=None, date_end=None, institution=None):
        Searches plenary protocols specified by the parameters
    search_activity(return_format="json", num=100, fid=None, date_start=None, date_end=None, documentID=None, plenaryprotocolID=None, institution=None):
        Searches activities specified by the parameters
    get_activity(btid, return_format="json"):
        Retrieves activities specified by IDs
    get_procedure(btid, return_format="json"):
        Retrieves procedures specified by IDs
    get_procedureposition(btid, return_format="json"):
        Retrieves procedure positions specified by IDs
    get_document(btid, return_format="json"):
        Retrieves documents specified by IDs
    get_person(btid, return_format="json"):
        Retrieves persons specified by IDs
    get_plenaryprotocol(btid, return_format="json"):
        Retrieves plenary protocols specified by IDs
    """


    def __init__(self, apikey=None):
        GEN_APIKEY = "I9FKdCn.hbfefNWCY336dL6x62vfwNKpoN2RZ1gp21"

        DATE_GEN_APIKEY = "31.05.2025"
        date_expiry = datetime.strptime(DATE_GEN_APIKEY, "%d.%m.%Y")

        today = datetime.now()
        if apikey is None and date_expiry.date() < today.date():
            logger.error("You need to supply your own API key.")
        elif apikey is None and date_expiry.date() > today.date():
            self.apikey = GEN_APIKEY
            logger.info("General API key used. It is valid until 31.05.2024.")
        elif apikey is not None:
            if not isinstance(apikey, str) and len(apikey) == 42:
                raise ValueError("No (correct) API key provided")
            else:
                self.apikey = apikey
                logger.debug("Personal API key is used.")

    def __str__(self):
        return "API key: "+str(self.apikey)

    def __repr__(self):
        return "API key: "+str(self.apikey)

    def query(self,
              resource,
              return_format="json",
              num=100,
              fid=None,
              date_start=None,
              date_end=None,
              updated_since=None,
              updated_until=None,
              institution=None,
              documentID=None,
              plenaryprotocolID=None,
              procedureID=None,
              descriptor=None,
              sachgebiet=None,
              document_type=None,
              title=None,):
        """A general search function for the official Bundestag API

        Parameters
            ----------
            resource: str
                The resource type to be queried. options are aktivitaet,
                drucksache, drucksache-text, person, plenarprotokoll,
                plenarprotokoll-text, vorgang or vorgangsposition
            return_format: str, optional
                Return format of the data. Defaults to json. XML not implemented
                yet. Other option is "object" which will return results as class
                objects
            num: int, optional
                Number of maximal results to be returned. Defaults to 100
            fid: int/list, optional
                ID of an entity. Can be a list to retrieve more than one entity
            date_start: str, optional
                Date after which entities should be retrieved. Format
                is "YYYY-MM-DD"
            date_end: str, optional
                Date before which entities should be retrieved. Format
                is "YYYY-MM-DD"
            updated_since: str, optional
                Date and time after which updated documents are to be retrieved
            updated_until: str, optional
                Date and time until which updated documents are to be retrieved
            institution: str, optional
                Filter results by institution BT, BR, BV or EK
            documentID: int, optional
                Entity ID of a document. Can be used to select activities,
                procedures and procedure positions that are connected to the
                document
            plenaryprotocolID: int, optional
                Entity ID of a plenary protocol. Can be used to select activities,
                procedures and procedure positions that are connected to the
                protocol
            procedureID: int, optional
                Entity ID of a process. Can be used to select procedure positions
                that are connected to the process
            descriptor: str/list, optional
                Keyword that is connected to the entities. Multiple strings can
                be supplied as a list but they will be joined via AND. An OR-
                search is not possible
            sachgebiet: str/list, optional
                Political field that is connected to the entities. Multiple 
                strings can be supplied as a list but they will be joined via
                AND. An OR-search is not possible
            document_type: str, optional
                The type of document to be returned.
            title: str/list, optional
                Keyword that can be found in the title of documents. Multiple 
                strings can be supplied as a list and will be joined via
                an OR-search.
        """

        BASE_URL = "https://search.dip.bundestag.de/api/v1/"
        RESOURCETYPES = ["aktivitaet", "drucksache", "drucksache-text", "person",
                         "plenarprotokoll", "plenarprotokoll-text", "vorgang",
                         "vorgangsposition"]
        INSTITUTIONS = ["BT", "BR", "BV", "EK"]
        if isinstance(resource, str) is True:
            resource = resource.lower()
        if resource not in RESOURCETYPES:
            raise ValueError("No or wrong resource")
        # Validate fid
        if isinstance(fid, list) is True:
            if all(isinstance(item, int) for item in fid) is False:
                try:
                    fid = [int(item) for item in fid]
                except ValueError as e:
                    raise Exception("IDs must be integers: {}".format(e)) from None
            else:
                fid = '&f.id='.join(map(str, fid))
        elif fid is not None:
            if not isinstance(fid, int):
                try:
                    fid = int(fid)
                except ValueError as e:
                    raise Exception("IDs must be integers: {}".format(e)) from None
        if return_format not in ["json", "xml", "object", "pandas"]:
            raise ValueError("return_format: Not a correct format!")
        if institution is not None and institution not in INSTITUTIONS:
            raise ValueError("Unknown institution")
        if resource not in ["aktivitaet", "vorgang", "vorgangsposition"]:
            if documentID is not None:
                raise ValueError(
                    "documentID must be combined with resource 'aktivitaet', 'vorgang' or 'vorgangsposition'")
            if plenaryprotocolID is not None:
                raise ValueError(
                    "plenaryprotocolID must be combined with resource 'aktivitaet', 'vorgang' or 'vorgangsposition'")
        elif resource in ["aktivitaet", "vorgang", "vorgangsposition"]:
            if documentID is not None and not isinstance(documentID, int):
                raise ValueError("documentID must be an integer")
            if plenaryprotocolID is not None and not isinstance(plenaryprotocolID, int):
                raise ValueError("plenaryprotocolID must be an integer")
        if resource not in ["vorgangsposition"]:
            if procedureID is not None:
                raise ValueError(
                    "procedureID must be combined with resource 'vorgangsposition'")
        elif resource in ["vorgangsposition"]:
            if procedureID is not None and not isinstance(procedureID, int):
                raise ValueError("procedureID must be an integer")
        if resource in ["drucksache", "drucksache-text", "vorgang", "vorgangsposition"]:
            if title is not None:
                if isinstance(title, list) is True:
                    if all(isinstance(item, str) for item in title) is False:
                        raise ValueError("All sachgebiet items need to be of type string.")
                    elif all(len(item) < 100 for item in title) is False:
                        raise ValueError("Strings are over 100 characters in length.")
                    else:
                        title = '&f.sachgebiet='.join(map(str, title))
                elif isinstance(title, str) is True:
                    if len(title) < 100:
                        raise ValueError("String is over 100 characters in length.")
                else:
                    raise ValueError("Sachgebiet must be string or a list of strings.")
            if document_type is not None:
                if isinstance(document_type, str) is False:
                    raise ValueError("document_type must be a string.")
        if resource not in ["drucksache", "drucksache-text", "vorgang", "vorgangsposition"]:
            if title is not None:
                raise ValueError("Title must be combined with a document of process")
            if document_type is not None:
                raise ValueError("Document type must be combined with a document")
        # Validate that only one of the possible IDs is given and raise an error otherwise
        non_none_count = sum(arg is not None for arg in [
                             plenaryprotocolID, documentID, procedureID])
        if non_none_count > 1:
            raise ValueError(
                "Can't select more than one of documentID, plenaryprotocolID and procedureID")
        # Validate the num parameter is an integer and positive
        if not isinstance(num, int) or num <= 0:
            raise ValueError("num must be an integer larger than zero")
        # Validate updated_since and updated_until are both in ISO 8601 format
        if updated_since is not None:
            if is_iso8601(updated_since) != True:
                raise ValueError(
                    "updated_since must be a string in the following format '2022-06-24T09:45:00'")
        if updated_until is not None:
            if is_iso8601(updated_until) != True:
                raise ValueError(
                    "updated_until must be a string in the following format '2022-06-24T09:45:00'")
        # Validate descriptors
        if descriptor is not None:
            if isinstance(descriptor, list) is True:
                if all(isinstance(item, str) for item in descriptor) is False:
                    raise ValueError("All descriptor items need to be of type string.")
                elif all(len(item) < 100 for item in descriptor) is False:
                    raise ValueError("Strings are over 100 characters in length.")
                else:
                    descriptor = '&f.deskriptor='.join(map(str, descriptor))
            elif isinstance(descriptor, str) is True:
                if len(descriptor) < 100:
                    raise ValueError("String is over 100 characters in length.")
            else:
                raise ValueError("Descriptor must be string or a list of strings.")
        # Validate sachgebiet
        if sachgebiet is not None:
            if isinstance(sachgebiet, list) is True:
                if all(isinstance(item, str) for item in sachgebiet) is False:
                    raise ValueError("All sachgebiet items need to be of type string.")
                elif all(len(item) < 100 for item in sachgebiet) is False:
                    raise ValueError("Strings are over 100 characters in length.")
                else:
                    sachgebiet = '&f.sachgebiet='.join(map(str, sachgebiet))
            elif isinstance(sachgebiet, str) is True:
                if len(sachgebiet) < 100:
                    raise ValueError("String is over 100 characters in length.")
            else:
                raise ValueError("Sachgebiet must be string or a list of strings.")

        r_url = BASE_URL+resource
        return_object = False
        if return_format == "object":
            return_format = "json"
            return_object = True

        payload = {"apikey": self.apikey,
                   "format": return_format,
                   "f.id": fid,
                   "f.datum.start": date_start,
                   "f.datum.end": date_end,
                   "f.aktualisiert.start": updated_since,
                   "f.aktualisiert.end": updated_until,
                   "f.drucksache": documentID,
                   "f.plenarprotokoll": plenaryprotocolID,
                   "f.vorgang": procedureID,
                   "f.zuordnung": institution,
                   "f.deskriptor": descriptor,
                   "f.sachgebiet": sachgebiet,
                   "f.drucksachetyp": document_type,
                   "f.titel": title,
                   "cursor": None}
        data = []
        prs = True
        while prs is True:
            r = requests.get(r_url, params=payload)
            logger.debug(r.url)
            if r.status_code == requests.codes.ok:
                content = r.json()
                if content["numFound"] == 0:
                    logging.info("No data was returned.")
                    data = "No data was returned."
                    prs = False
                elif content["numFound"] > 0 and content["numFound"] <= 50:
                    data.extend(content["documents"])
                    prs = False
                elif content["numFound"] > 50:
                    if payload["cursor"] == content["cursor"]:
                        prs = False
                    data.extend(content["documents"])
                    if num is not None and len(data) >= num:
                        data = data[0:num]
                        prs = False
                    payload["cursor"] = content["cursor"]
            elif r.status_code == 400:
                logger.error("A syntax error occured. Code {code}: {message}".format(
                    code=r.status_code, message=r.reason))
            elif r.status_code == 401:
                logger.error("An authorization error occured. Likely an error with you API key. Code {code}: {message}".format(
                    code=r.status_code, message=r.reason))
            elif r.status_code == 404:
                logger.error("The API is not reachable. Code {code}: {message}".format(
                    code=r.status_code, message=r.reason))
            else:
                logger.error("An error occured. Code {code}: {message}".format(
                    code=r.status_code, message=r.reason))
        if return_object is True:
            if resource == "aktivitaet":
                data = {name["id"]: Aktivitaet(name) for name in data}
            elif resource == "drucksache":
                data = {name["id"]: Drucksache(name) for name in data}
            elif resource == "drucksache-text":
                data = {name["id"]: Drucksache(name) for name in data}
            elif resource == "person":
                data = {name["id"]: Person(name) for name in data}
            elif resource == "plenarprotokoll":
                data = {name["id"]: Plenarprotokoll(name) for name in data}
            elif resource == "plenarprotokoll-text":
                data = {name["id"]: Plenarprotokoll(name) for name in data}
            elif resource == "vorgang":
                data = {name["id"]: Vorgang(name) for name in data}
            elif resource == "vorgangsposition":
                data = {name["id"]: Vorgangsposition(name) for name in data}
        if return_format == "pandas":
            import pandas as pd
            data = pd.json_normalize(data)
        if len(data) == 1 and isinstance(data, dict):
            tl = list(data.keys())
            data = data[tl[0]]
        elif len(data) == 1 and isinstance(data, list):
            data = data[0]
        return data

    def search_procedure(self,
                         return_format="json",
                         num=100,
                         fid=None,
                         date_start=None,
                         date_end=None,
                         updated_since=None,
                         updated_until=None,
                         descriptor=None,
                         sachgebiet=None,
                         document_type=None,
                         title=None,):
        """
        Searches procedures specified by the parameters

        Parameters
        ----------
        return_format: str, optional
            Return format of the data. Defaults to json. XML not implemented
            yet. Other option is "object" which will return results as class
            objects
        num: int, optional
            Number of maximal results to be returned. Defaults to 100
        fid: int/list, optional
            ID of a procedure entity. Can be a list to retrieve more than
            one entity
        date_start: str, optional
            Date after which entities should be retrieved. Format
            is "YYYY-MM-DD"
        date_end: str, optional
            Date before which entities should be retrieved. Format
            is "YYYY-MM-DD"
        updated_since: str, optional
            Date and time after which updated documents are to be retrieved
        updated_until: str, optional
            Date and time until which updated documents are to be retrieved
        descriptor: str/list, optional
            Keyword that is connected to the entities. Multiple strings can
            be supplied as a list but they will be joined via AND. An OR-
            search is not possible         
        sachgebiet: str/list, optional
            Political field that is connected to the entities. Multiple 
            strings can be supplied as a list but they will be joined via
            AND. An OR-search is not possible
        document_type: str, optional
            The type of document to be returned.
        title: str/list, optional
            Keyword that can be found in the title of documents. Multiple 
            strings can be supplied as a list and will be joined via
            an OR-search.

        Returns
        -------
        data: list
            a list of dictionaries or class objects of procedures

        """
        data = self.query(resource="vorgang",
                          return_format=return_format,
                          fid=fid,
                          date_start=date_start,
                          date_end=date_end,
                          num=num,
                          updated_since=updated_since,
                          updated_until=updated_until,
                          descriptor=descriptor,
                          sachgebiet=sachgebiet,
                          document_type=document_type,
                          title=title)
        return data

    def get_procedure(self,
                      btid=None,
                      return_format="json",
                      documentID=None,
                      plenaryprotocolID=None):
        """
        Retrieves procedures specified by IDs

        Parameters
        ----------
        btid: int/list, optional
            ID of a procedure entity. Can be a list to retrieve more than
            one entity
        return_format: str, optional
            Return format of the data. Defaults to json. XML not implemented
            yet. Other option is "object" which will return results as class
            objects
        documentID: int, optional
            Returns procedure related to that documentID
        plenaryprotocolID: int, optional
            Entity ID of a plenary protocol. Can be used to select activities,
            procedures and procedure positions that are connected to the
            protocol

        Returns
        -------
        data: list
            a list of dictionaries or class objects of procedures
        """
        if btid is None and documentID is None and plenaryprotocolID is None:
            raise ValueError(
                "Either an procedure ID, document ID or plenary protocol ID must be supplied")
        else:
            data = self.query(resource="vorgang",
                              fid=btid,
                              return_format=return_format,
                              documentID=documentID,
                              plenaryprotocolID=plenaryprotocolID)
            return data

    def search_procedureposition(self,
                                 return_format="json",
                                 num=100,
                                 fid=None,
                                 date_start=None,
                                 date_end=None,
                                 updated_since=None,
                                 updated_until=None,
                                 document_type=None,
                                 title=None,):
        """
        Searches procedure positions specified by the parameters

        Parameters
        ----------
        return_format: str, optional
            Return format of the data. Defaults to json. XML not implemented
            yet. Other option is "object" which will return results as class
            objects
        num: int, optional
            Number of maximal results to be returned. Defaults to 100
        fid: int/list, optional
            ID of an procedure position entity. Can be a list to retrieve more
            than one entity
        date_start: str, optional
            Date after which entities should be retrieved. Format
            is "YYYY-MM-DD"
        date_end: str, optional
            Date before which entities should be retrieved. Format
            is "YYYY-MM-DD"
        updated_since: str, optional
            Date and time after which updated documents are to be retrieved
        updated_until: str, optional
            Date and time until which updated documents are to be retrieved
        document_type: str, optional
            The type of document to be returned.
        title: str/list, optional
            Keyword that can be found in the title of documents. Multiple 
            strings can be supplied as a list and will be joined via
            an OR-search.

        Returns
        -------
        data: list
            a list of dictionaries or class objects of procedure positions
        """

        data = self.query(resource="vorgangsposition",
                          return_format=return_format,
                          fid=fid,
                          date_start=date_start,
                          date_end=date_end,
                          num=num,
                          updated_since=updated_since,
                          updated_until=updated_until,
                          document_type=document_type,
                          title=title)
        return data

    def get_procedureposition(self,
                              btid,
                              return_format="json",
                              documentID=None,
                              procedureID=None,
                              plenaryprotocolID=None):
        """
        Retrieves procedure positions specified by IDs

        Parameters
        ----------
        btid: int/list, optional
            ID of a procedure position entity. Can be a list to retrieve more than
            one entity
        return_format: str, optional
            Return format of the data. Defaults to json. XML not implemented
            yet. Other option is "object" which will return results as class
            objects
        documentID: int, optional
            Returns procedure positions related to that documentID
        procedureID: int, optional
            Returns procedure positions related to that procedure
        plenaryprotocolID: int, optional
            Entity ID of a plenary protocol. Can be used to select activities,
            procedures and procedure positions that are connected to the
            protocol

        Returns
        -------
        data: list
            a list of dictionaries or class objects of procedure positions
        """
        if btid is None and documentID is None and procedureID is None:
            raise ValueError(
                "Either an procedure ID, document ID or procedure ID must be supplied")
        else:
            data = self.query(resource="vorgangsposition",
                              fid=btid,
                              return_format=return_format,
                              documentID=documentID,
                              procedureID=procedureID,
                              plenaryprotocolID=plenaryprotocolID)
            return data

    def search_document(self,
                        return_format="json",
                        num=100,
                        fid=None,
                        date_start=None,
                        date_end=None,
                        institution=None,
                        fulltext=False,
                        updated_since=None,
                        updated_until=None,
                        document_type=None,
                        title=None,):
        """
        Searches documents specified by the parameters

        Parameters
        ----------
        return_format: str, optional
            Return format of the data. Defaults to json. XML not implemented
            yet. Other option is "object" which will return results as class
            objects
        num: int, optional
            Number of maximal results to be returned. Defaults to 100
        fid: int/list, optional
            ID of a document entity. Can be a list to retrieve more than one entity
        date_start: str, optional
            Date after which entities should be retrieved. Format
            is "YYYY-MM-DD"
        date_end: str, optional
            Date before which entities should be retrieved. Format
            is "YYYY-MM-DD"
        institution: str, optional
            Filter results by institution BT, BR, BV or EK
        fulltext: boolean
            Whether the fulltext (if available) should be requested or not. Default is false
        updated_since: str, optional
            Date and time after which updated documents are to be retrieved
        updated_until: str, optional
            Date and time until which updated documents are to be retrieved
        document_type: str, optional
            The type of document to be returned.
        title: str/list, optional
            Keyword that can be found in the title of documents. Multiple 
            strings can be supplied as a list and will be joined via
            an OR-search.

        Returns
        -------
        data: list
            a list of dictionaries or class objects of documents
        """

        if fulltext == False:
            resource = "drucksache"
        elif fulltext == True:
            resource = "drucksache-text"
        else:
            resource = "drucksache"

        data = self.query(resource=resource,
                          return_format=return_format,
                          fid=fid,
                          date_start=date_start,
                          date_end=date_end,
                          institution=institution,
                          num=num,
                          updated_since=updated_since,
                          updated_until=updated_until,
                          document_type=document_type,
                          title=title)
        return data

    def get_document(self,
                     btid,
                     return_format="json",
                     fulltext=False):
        """
        Retrieves documents specified by IDs

        Parameters
        ----------
        btid: int/list
            ID of a document entity. Can be a list to retrieve more than
            one entity
        return_format: str, optional
            Return format of the data. Defaults to json. XML not implemented
            yet. Other option is "object" which will return results as class
            objects
        fulltext: boolean
            Whether the fulltext (if available) should be requested or not. Default is False    

        Returns
        -------
        data: list
            a list of dictionaries or class objects of documents
        """

        if fulltext == False:
            resource = "drucksache"
        elif fulltext == True:
            resource = "drucksache-text"
        else:
            resource = "drucksache"

        data = self.query(resource=resource,
                          fid=btid,
                          return_format=return_format)
        return data

    def search_person(self,
                      return_format="json",
                      num=100,
                      updated_since=None,
                      updated_until=None,):
        """
        Searches persons specified by the parameters

        Parameters
        ----------
        return_format: str, optional
            Return format of the data. Defaults to json. XML not implemented
            yet. Other option is "object" which will return results as class
            objects
        num: int, optional
            Number of maximal results to be returned. Defaults to 100
        updated_since: str, optional
            Date and time after which updated documents are to be retrieved
        updated_until: str, optional
            Date and time until which updated documents are to be retrieved

        Returns
        -------
        data: list
            a list of dictionaries or class objects of persons
        """

        data = self.query(resource="person",
                          return_format=return_format,
                          num=num,
                          updated_since=updated_since,
                          updated_until=updated_until,)
        return data

    def get_person(self,
                   btid,
                   return_format="json"):
        """
        Retrieves persons specified by IDs

        Parameters
        ----------
        btid: int/list
            ID of a person entity. Can be a list to retrieve more than
            one entity
        return_format: str, optional
            Return format of the data. Defaults to json. XML not implemented
            yet. Other option is "object" which will return results as class
            objects

        Returns
        -------
        data: list
            a list of dictionaries or class objects of persons
        """

        data = self.query(resource="person",
                          fid=btid,
                          return_format=return_format)
        return data

    def search_plenaryprotocol(self,
                               return_format="json",
                               num=100,
                               **kwargs):
        """
        Searches plenary protocols specified by the parameters

        Parameters
        ----------
        return_format: str, optional
            Return format of the data. Defaults to json. XML not implemented
            yet. Other option is "object" which will return results as class
            objects
        num: int, optional
            Number of maximal results to be returned. Defaults to 100
        date_start: str, optional
            Date after which entities should be retrieved. Format
            is "YYYY-MM-DD"
        date_end: str, optional
            Date before which entities should be retrieved. Format
            is "YYYY-MM-DD"
        institution: str, optional
            Filter results by institution BT, BR, BV or EK
        fulltext: boolean
            Whether the fulltext (if available) should be requested or not. Default is false
        updated_since: str, optional
            Date and time after which updated documents are to be retrieved
        updated_until: str, optional
            Date and time until which updated documents are to be retrieved

        Returns
        -------
        data: list
            a list of dictionaries or class objects of plenary protocols
        """

        date_start = kwargs.get("date_start")
        date_end = kwargs.get("date_end")
        institution = kwargs.get("institution")
        fulltext = kwargs.get("fulltext")
        updated_since = kwargs.get("updated_since")
        updated_until = kwargs.get("updated_until")

        if fulltext == False:
            resource = "plenarprotokoll"
        elif fulltext == True:
            resource = "plenarprotokoll-text"
        else:
            resource = "plenarprotokoll"

        data = self.query(resource=resource,
                          return_format=return_format,
                          date_start=date_start,
                          date_end=date_end,
                          institution=institution,
                          num=num,
                          updated_since=updated_since,
                          updated_until=updated_until,)

        return data

    def get_plenaryprotocol(self,
                            btid,
                            return_format="json",
                            fulltext=False):
        """
        Retrieves plenary protocols specified by IDs

        Parameters
        ----------
        btid: int/list
            ID of a plenary protocol entity. Can be a list to retrieve more than
            one entity
        return_format: str, optional
            Return format of the data. Defaults to json. XML not implemented
            yet. Other option is "object" which will return results as class
            objects
        fulltext: boolean
            Whether the fulltext (if available) should be requested or not. Default is false

        Returns
        -------
        data: list
            a list of dictionaries or class objects of plenary protocols
        """

        if fulltext == False:
            resource = "plenarprotokoll"
        elif fulltext == True:
            resource = "plenarprotokoll-text"
        else:
            resource = "plenarprotokoll"

        data = self.query(resource=resource,
                          fid=btid,
                          return_format=return_format)
        return data

    def search_activity(self,
                        return_format="json",
                        num=100,
                        date_start=None,
                        date_end=None,
                        institution=None,
                        updated_since=None,
                        updated_until=None,
                        descriptor=None,):
        """
        Searches activities specified by the parameters

        Parameters
        ----------
        return_format: str, optional
            Return format of the data. Defaults to json. XML not implemented
            yet. Other option is "object" which will return results as class
            objects
        num: int, optional
            Number of maximal results to be returned. Defaults to 100
        date_start: str, optional
            Date after which entities should be retrieved. Format
            is "YYYY-MM-DD"
        date_end: str, optional
            Date before which entities should be retrieved. Format
            is "YYYY-MM-DD"
        institution: str, optional
            Filter results by institution BT, BR, BV or EK
        updated_since: str, optional
            Date and time after which updated documents are to be retrieved
        updated_until: str, optional
            Date and time until which updated documents are to be retrieved
        descriptor: str/list, optional
            Keyword that is connected to the entities. Multiple strings can
            be supplied as a list but they will be joined via AND. An OR-
            search is not possible         

        Returns
        -------
        data: list
            a list of dictionaries or class objects of activities
        """

        data = self.query(resource="aktivitaet",
                          return_format=return_format,
                          date_start=date_start,
                          date_end=date_end,
                          institution=institution,
                          num=num,
                          updated_since=updated_since,
                          updated_until=updated_until,
                          descriptor=descriptor)

        return data

    def get_activity(self,
                     btid=None,
                     return_format="json",
                     documentID=None,
                     plenaryprotocolID=None):
        """
        Retrieves activities specified by IDs

        Parameters
        ----------
        btid: int/list, optional
            ID of an activity entity. Can be a list to retrieve more than
            one entity
        return_format: str, optional
            Return format of the data. Defaults to json. XML not implemented
            yet. Other option is "object" which will return results as class
            objects
        documentID: int, optional
            Entity ID of a document. Can be used to select activities,
            procedures and procedure positions that are connected to the
            document
        plenaryprotocolID: int, optional
            Entity ID of a plenary protocol. Can be used to select activities,
            procedures and procedure positions that are connected to the
            protocol
        Returns
        -------
        data: list
            a list of dictionaries or class objects of activities
        """
        if btid is None and documentID is None and plenaryprotocolID is None:
            raise ValueError(
                "Either an activity ID, document ID or plenary protocol ID must be supplied")
        else:
            data = self.query(resource="aktivitaet",
                              fid=btid,
                              return_format=return_format,
                              documentID=documentID,
                              plenaryprotocolID=plenaryprotocolID)
            return data

    def list_methods(self):
        """
        list all methods offered by btaConnection

        Returns
        -------
        data: list
            A list of strings

        """
        list_of_methods = dir(btaConnection)
        list_of_methods = [item for item in list_of_methods if "__" not in item]
        return list_of_methods



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

from pathlib import Path
this_directory = Path("__file__").parent
long_description = (this_directory / "README.md").read_text()

setup(
    author='Julian Schibberges',
    author_email="julian@schibberges.de",
    description='Python wrapper for the official Bundestag-API',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url="https://github.com/jschibberges/Bundestag-API",
    readme="README.md",
    license="MIT",
    name='bundestag_api',
    version='1.0.8',
    packages=find_packages(),
    install_requires=[
         'requests>=2.0.0',
    ],
    python_requires='>=3.7.0'
)
